@@ -295,6 +295,15 @@ class DBaaSAPINotification(object):
     '''
 
     event_type_format = 'dbaas.%s.%s'
+    notify_callback = None
+
+    @classmethod
+    def register_notify_callback(cls, callback):
+        """A callback registered here will be fired whenever
+        a notification is sent out. The callback should
+        take a notification object, and event_qualifier.
+        """
+        cls.notify_callback = callback
 
     @abc.abstractmethod
     def event_type(self):
@@ -324,7 +333,7 @@ class DBaaSAPINotification(object):
 
     def optional_error_traits(self):
         'Returns list of optional traits for error notification'
-        return []
+        return ['instance_id']
 
     def required_base_traits(self):
         return ['tenant_id', 'client_ip', 'server_ip', 'server_type',
@@ -338,6 +347,10 @@ class DBaaSAPINotification(object):
     def server_type(self, server_type):
         self.payload['server_type'] = server_type
 
+    @property
+    def request_id(self):
+        return self.payload['request_id']
+
     def __init__(self, context, **kwargs):
         self.context = context
         self.needs_end_notification = True
@@ -395,6 +408,8 @@ class DBaaSAPINotification(object):
         del context.notification
         notifier = rpc.get_notifier(service=self.payload['server_type'])
         notifier.info(context, qualified_event_type, self.payload)
+        if self.notify_callback:
+            self.notify_callback(event_qualifier)
 
     def notify_start(self, **kwargs):
         self._notify('start', self.required_start_traits(),
@@ -491,6 +506,16 @@ class DBaaSInstanceDelete(DBaaSAPINotification):
         return ['instance_id']
 
 
+class DBaaSInstanceResetStatus(DBaaSAPINotification):
+
+    def event_type(self):
+        return 'instance_reset_status'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['instance_id']
+
+
 class DBaaSInstanceDetach(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -550,6 +575,17 @@ class DBaaSClusterDelete(DBaaSAPINotification):
         return ['cluster_id']
 
 
+class DBaaSClusterResetStatus(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'cluster_reset_status'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['cluster_id']
+
+
 class DBaaSClusterAddShard(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -742,3 +778,14 @@ class DBaaSConfigurationEdit(DBaaSAPINotification):
     @abc.abstractmethod
     def required_start_traits(self):
         return ['configuration_id']
+
+
+class DBaaSInstanceUpgrade(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'upgrade'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['instance_id', 'datastore_version_id']
