@@ -59,7 +59,7 @@ def read_file(path, codec=IdentityCodec(), as_root=False, decode=True):
         if as_root:
             return _read_file_as_root(path, codec, decode=decode)
 
-        with open(path, 'rb') as fp:
+        with open(path, 'r') as fp:
             if decode:
                 return codec.deserialize(fp.read())
             return codec.serialize(fp.read())
@@ -110,7 +110,7 @@ def _read_file_as_root(path, codec, decode=True):
     :type decode:              boolean
     """
     with tempfile.NamedTemporaryFile() as fp:
-        copy(path, fp.name, force=True, as_root=True)
+        copy(path, fp.name, force=True, dereference=True, as_root=True)
         chmod(fp.name, FileMode.ADD_READ_ALL(), as_root=True)
         if decode:
             return codec.deserialize(fp.read())
@@ -144,11 +144,12 @@ def write_file(path, data, codec=IdentityCodec(), as_root=False, encode=True):
         if as_root:
             _write_file_as_root(path, data, codec, encode=encode)
         else:
-            with open(path, 'wb', 0) as fp:
+            with open(path, 'w') as fp:
                 if encode:
                     fp.write(codec.serialize(data))
                 else:
                     fp.write(codec.deserialize(data))
+                fp.flush()
     else:
         raise exception.UnprocessableEntity(_("Invalid path: %s") % path)
 
@@ -170,11 +171,12 @@ def _write_file_as_root(path, data, codec, encode=True):
     """
     # The files gets removed automatically once the managing object goes
     # out of scope.
-    with tempfile.NamedTemporaryFile('wb', 0, delete=False) as fp:
+    with tempfile.NamedTemporaryFile('w', delete=False) as fp:
         if encode:
             fp.write(codec.serialize(data))
         else:
             fp.write(codec.deserialize(data))
+        fp.flush()
         fp.close()  # Release the resource before proceeding.
         copy(fp.name, path, force=True, as_root=True)
 
@@ -228,6 +230,10 @@ class FileMode(object):
     def SET_USR_RW(cls):
         return cls(reset=[stat.S_IRUSR | stat.S_IWUSR])  # =0600
 
+    @classmethod
+    def SET_USR_RWX(cls):
+        return cls(reset=[stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR])  # =0700
+
     @classmethod
     def ADD_ALL_R(cls):
         return cls(add=[stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH])  # +0444
@@ -687,7 +693,7 @@ def move(source, destination, force=False, **kwargs):
 
 
 def copy(source, destination, force=False, preserve=False, recursive=True,
-         **kwargs):
+         dereference=False, **kwargs):
     """Copy a given file or directory to another location.
     Copy does NOT attempt to preserve ownership, permissions and timestamps
     unless the 'preserve' option is enabled.
@@ -710,6 +716,9 @@ def copy(source, destination, force=False, preserve=False, recursive=True,
     :param recursive:       Copy directories recursively.
     :type recursive:        boolean
 
+    :param dereference:     Follow symbolic links when copying from them.
+    :type dereference:      boolean
+
     :raises:                :class:`UnprocessableEntity` if source or
                             destination not given.
     """
@@ -719,7 +728,8 @@ def copy(source, destination, force=False, preserve=False, recursive=True,
     elif not destination:
         raise exception.UnprocessableEntity(_("Missing destination path."))
 
-    options = (('f', force), ('p', preserve), ('R', recursive))
+    options = (('f', force), ('p', preserve), ('R', recursive),
+               ('L', dereference))
     _execute_shell_cmd('cp', options, source, destination, **kwargs)
 
 
