@@ -18,6 +18,7 @@ import uuid
 
 from cinderclient import exceptions as cinder_exceptions
 import cinderclient.v2.client as cinderclient
+from cinderclient.v2 import volumes as cinderclient_volumes
 from mock import Mock, MagicMock, patch, PropertyMock, call
 from novaclient import exceptions as nova_exceptions
 import novaclient.v2.flavors
@@ -60,9 +61,10 @@ VOLUME_ID = 'volume-id-1'
 
 class FakeOptGroup(object):
     def __init__(self, tcp_ports=['3306', '3301-3307'],
-                 udp_ports=[]):
+                 udp_ports=[], icmp=False):
         self.tcp_ports = tcp_ports
         self.udp_ports = udp_ports
+        self.icmp = icmp
 
 
 class fake_Server:
@@ -81,7 +83,8 @@ class fake_Server:
 class fake_ServerManager:
     def create(self, name, image_id, flavor_id, files, userdata,
                security_groups, block_device_mapping, availability_zone=None,
-               nics=None, config_drive=False):
+               nics=None, config_drive=False,
+               scheduler_hints=None):
         server = fake_Server()
         server.id = "server_id"
         server.name = name
@@ -193,10 +196,11 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
         self.orig_DBI_find_by = DBInstance.find_by
         self.userdata = "hello moto"
         self.guestconfig_content = "guest config"
-        with NamedTemporaryFile(suffix=".cloudinit", delete=False) as f:
+        with NamedTemporaryFile(mode="w", suffix=".cloudinit",
+                                delete=False) as f:
             self.cloudinit = f.name
             f.write(self.userdata)
-        with NamedTemporaryFile(delete=False) as f:
+        with NamedTemporaryFile(mode="w", delete=False) as f:
             self.guestconfig = f.name
             f.write(self.guestconfig_content)
         self.freshinstancetasks = taskmanager_models.FreshInstanceTasks(
@@ -214,6 +218,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
         self.task_models_conf_patch = patch('trove.taskmanager.models.CONF')
         self.task_models_conf_mock = self.task_models_conf_patch.start()
         self.addCleanup(self.task_models_conf_patch.stop)
+        self.inst_models_conf_patch = patch('trove.instance.models.CONF')
+        self.inst_models_conf_mock = self.inst_models_conf_patch.start()
+        self.addCleanup(self.inst_models_conf_patch.stop)
 
     def tearDown(self):
         super(FreshInstanceTasksTest, self).tearDown()
@@ -249,9 +256,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             else:
                 return ''
 
-        self.task_models_conf_mock.get.side_effect = fake_conf_getter
+        self.inst_models_conf_mock.get.side_effect = fake_conf_getter
         # execute
-        files = self.freshinstancetasks._get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test")
         # verify
         self.assertTrue(
             '/etc/trove/conf.d/guest_info.conf' in files)
@@ -272,9 +279,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             else:
                 return ''
 
-        self.task_models_conf_mock.get.side_effect = fake_conf_getter
+        self.inst_models_conf_mock.get.side_effect = fake_conf_getter
         # execute
-        files = self.freshinstancetasks._get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test")
         # verify
         self.assertTrue(
             '/etc/guest_info' in files)
@@ -366,6 +373,15 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                           self.freshinstancetasks._create_secgroup,
                           datastore_manager)
 
+    def test_create_sg_rules_icmp(self):
+        datastore_manager = 'mysql'
+        self.task_models_conf_mock.get = Mock(
+            return_value=FakeOptGroup(icmp=True))
+        self.freshinstancetasks.update_db = Mock()
+        self.freshinstancetasks._create_secgroup(datastore_manager)
+        self.assertEqual(3, taskmanager_models.SecurityGroupRule.
+                         create_sec_group_rule.call_count)
+
     @patch.object(BaseInstance, 'update_db')
     @patch('trove.taskmanager.models.CONF')
     @patch('trove.taskmanager.models.LOG')
@@ -379,18 +395,17 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             'Error creating security group for instance',
             self.freshinstancetasks.create_instance, mock_flavor,
             'mysql-image-id', None, None, 'mysql', 'mysql-server', 2,
-            None, None, None, None, Mock(), None, None, None, None)
+            None, None, None, None, Mock(), None, None, None, None, None)
 
     @patch.object(BaseInstance, 'update_db')
     @patch.object(backup_models.Backup, 'get_by_id')
     @patch.object(taskmanager_models.FreshInstanceTasks, 'report_root_enabled')
-    @patch.object(taskmanager_models.FreshInstanceTasks, '_get_injected_files')
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'get_injected_files')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_secgroup')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_build_volume_info')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_server')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_guest_prepare')
     @patch.object(template, 'SingleInstanceConfigTemplate')
-    @patch.object(template, 'OverrideConfigTemplate')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_dns_entry',
                   side_effect=TroveError)
     @patch('trove.taskmanager.models.LOG')
@@ -402,11 +417,11 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             self.freshinstancetasks.create_instance, mock_flavor,
             'mysql-image-id', None, None, 'mysql', 'mysql-server',
             2, Mock(), None, 'root_password', None, Mock(), None, None, None,
-            None)
+            None, None)
 
     @patch.object(BaseInstance, 'update_db')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_dns_entry')
-    @patch.object(taskmanager_models.FreshInstanceTasks, '_get_injected_files')
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'get_injected_files')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_server')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_secgroup')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_build_volume_info')
@@ -417,6 +432,8 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                              mock_guest_prepare,
                              mock_build_volume_info,
                              mock_create_secgroup,
+                             mock_create_server,
+                             mock_get_injected_files,
                              *args):
         mock_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
         config_content = {'config_contents': 'some junk'}
@@ -428,13 +445,18 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                                                 'mysql-server', 2,
                                                 None, None, None, None,
                                                 overrides, None, None,
-                                                'volume_type', None)
+                                                'volume_type', None,
+                                                {'group': 'sg-id'})
         mock_create_secgroup.assert_called_with('mysql')
         mock_build_volume_info.assert_called_with('mysql', volume_size=2,
                                                   volume_type='volume_type')
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
             config_content, None, overrides, None, None, None)
+        mock_create_server.assert_called_with(
+            8, 'mysql-image-id', mock_create_secgroup(),
+            'mysql', mock_build_volume_info()['block_device'], None,
+            None, mock_get_injected_files(), {'group': 'sg-id'})
 
     @patch.object(trove.guestagent.api.API, 'attach_replication_slave')
     @patch.object(rpc, 'get_client')
@@ -673,6 +695,10 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
                                                    True)
         stub_flavor_manager.get = MagicMock(return_value=nova_flavor)
 
+        self.instance_task._volume_client = MagicMock(spec=cinderclient)
+        self.instance_task._volume_client.volumes = Mock(
+            spec=cinderclient_volumes.VolumeManager)
+
         answers = (status for status in
                    self.get_inst_service_status('inst_stat-id',
                                                 [ServiceStatuses.SHUTDOWN,
@@ -682,7 +708,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
 
         def side_effect_func(*args, **kwargs):
             if 'instance_id' in kwargs:
-                return answers.next()
+                return next(answers)
             elif ('id' in kwargs and 'deleted' in kwargs
                   and not kwargs['deleted']):
                 return db_instance
@@ -781,16 +807,38 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
 
     @patch.object(BaseInstance, 'update_db')
     def test_detach_replica(self, mock_update_db):
-        self.instance_task.detach_replica(Mock(), True)
-        self.instance_task._guest.detach_replica.assert_called_with(True)
-        mock_update_db.assert_called_with(slave_of_id=None)
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            self.instance_task.detach_replica(Mock(), True)
+            self.instance_task._guest.detach_replica.assert_called_with(True)
+            mock_update_db.assert_called_with(slave_of_id=None)
+            tr_mock.assert_not_called()
 
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            self.instance_task.detach_replica(Mock(), False)
+            self.instance_task._guest.detach_replica.assert_called_with(False)
+            mock_update_db.assert_called_with(slave_of_id=None)
+            tr_mock.assert_called_once_with()
+
+    @patch.object(BaseInstance, 'update_db')
     @patch('trove.taskmanager.models.LOG')
-    def test_error_detach_replica(self, mock_logging):
-        with patch.object(self.instance_task._guest, 'detach_replica',
-                          side_effect=GuestError):
-            self.assertRaises(GuestError, self.instance_task.detach_replica,
-                              Mock(), True)
+    def test_error_detach_replica(self, mock_logging, mock_update_db):
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            with patch.object(self.instance_task._guest, 'detach_replica',
+                              side_effect=GuestError):
+                self.assertRaises(
+                    GuestError, self.instance_task.detach_replica,
+                    Mock(), True)
+                mock_update_db.assert_not_called()
+                tr_mock.assert_not_called()
+
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            with patch.object(self.instance_task._guest, 'detach_replica',
+                              side_effect=GuestError):
+                self.assertRaises(
+                    GuestError, self.instance_task.detach_replica,
+                    Mock(), False)
+                mock_update_db.assert_not_called()
+                tr_mock.assert_called_once_with()
 
     @patch.object(BaseInstance, 'update_db')
     def test_make_read_only(self, mock_update_db):
@@ -877,6 +925,36 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.instance_task.demote_replication_master()
         self.instance_task._guest.demote_replication_master.assert_any_call()
 
+    @patch.multiple(taskmanager_models.BuiltInstanceTasks,
+                    get_injected_files=Mock(return_value="the-files"))
+    def test_upgrade(self, *args):
+        pre_rebuild_server = self.instance_task.server
+        dsv = Mock(image_id='foo_image')
+        mock_volume = Mock(attachments=[{'device': '/dev/mock_dev'}])
+        with patch.object(self.instance_task._volume_client.volumes, "get",
+                          Mock(return_value=mock_volume)):
+            mock_server = Mock(status='ACTIVE')
+            with patch.object(self.instance_task._nova_client.servers,
+                              'get', Mock(return_value=mock_server)):
+                with patch.multiple(self.instance_task._guest,
+                                    pre_upgrade=Mock(return_value={}),
+                                    post_upgrade=Mock()):
+                    self.instance_task.upgrade(dsv)
+
+                    self.instance_task._guest.pre_upgrade.assert_called_with()
+                    pre_rebuild_server.rebuild.assert_called_with(
+                        dsv.image_id, files="the-files")
+                    self.instance_task._guest.post_upgrade.assert_called_with(
+                        mock_volume.attachments[0])
+
+    def test_fix_device_path(self):
+        self.assertEqual("/dev/vdb", self.instance_task.
+                         _fix_device_path("vdb"))
+        self.assertEqual("/dev/dev", self.instance_task.
+                         _fix_device_path("dev"))
+        self.assertEqual("/dev/vdb/dev", self.instance_task.
+                         _fix_device_path("vdb/dev"))
+
 
 class BackupTasksTest(trove_testtools.TestCase):
 
